@@ -139,12 +139,12 @@ class Config:
         # because they are dynamically resolved from the default_config. so we
         # need to ignore the errors here about missing attributes.
         for k in self.keys:
-            if k.key not in valid_keys:
+            if k.key.lower() not in valid_keys:
                 raise ConfigError("No such key: %s" % k.key)
             for m in k.modifiers:
-                if m not in valid_mods:
+                if m.lower() not in valid_mods:
                     raise ConfigError("No such modifier: %s" % m)
         for ms in self.mouse:
             for m in ms.modifiers:
-                if m not in valid_mods:
+                if m.lower() not in valid_mods:
                     raise ConfigError("No such modifier: %s" % m)
