@@ -39,7 +39,7 @@ from libqtile.command import interface
 from libqtile.command.base import CommandError, CommandException, CommandObject
 from libqtile.command.client import InteractiveCommandClient
 from libqtile.command.interface import IPCCommandServer, QtileCommandInterface
-from libqtile.config import Click, Drag, Key, KeyChord, Match, Rule
+from libqtile.config import Click, Drag, Key, KeyChord, Match, Mouse, Rule
 from libqtile.config import ScratchPad as ScratchPadConfig
 from libqtile.config import Screen
 from libqtile.core.lifecycle import lifecycle
@@ -50,7 +50,7 @@ from libqtile.extension.base import _Extension
 from libqtile.group import _Group
 from libqtile.log_utils import logger
 from libqtile.scratchpad import ScratchPad
-from libqtile.utils import get_cache_dir, lget, send_notification
+from libqtile.utils import get_cache_dir, lget, send_notification, subscribe_for_resume_events
 from libqtile.widget.base import _Widget
 
 if TYPE_CHECKING:
@@ -59,6 +59,7 @@ if TYPE_CHECKING:
     from typing_extensions import Literal
 
     from libqtile.command.base import ItemT
+    from libqtile.confreader import Config
     from libqtile.layout.base import Layout
 
 
@@ -72,11 +73,11 @@ class Qtile(CommandObject):
     def __init__(
         self,
         kore: base.Core,
-        config,  # mypy doesn't like the config's dynamic attributes
+        config: Config,
         no_spawn: bool = False,
         state: str | None = None,
         socket_path: str | None = None,
-    ):
+    ) -> None:
         self.core = kore
         self.config = config
         self.no_spawn = no_spawn
@@ -84,7 +85,7 @@ class Qtile(CommandObject):
         self.socket_path = socket_path
 
         self._drag: tuple | None = None
-        self.mouse_map: dict[int, list[Click | Drag]] = {}
+        self.mouse_map: dict[int, list[Mouse]] = {}
 
         self.windows_map: dict[int, base.WindowType] = {}
         self.widgets_map: dict[str, _Widget] = {}
@@ -103,11 +104,12 @@ class Qtile(CommandObject):
 
         self.server = IPCCommandServer(self)
 
-    def load_config(self, initial=False) -> None:
+    def load_config(self, initial: bool = False) -> None:
         try:
             self.config.load()
             self.config.validate()
         except Exception as e:
+            logger.exception("Configuration error:")
             send_notification("Configuration error", str(e))
 
         if hasattr(self.core, "wmname"):
@@ -160,7 +162,7 @@ class Qtile(CommandObject):
             else:
                 self._state.apply(self)
 
-        self.core.distribute_windows(initial)
+        self.core.on_config_load(initial)
 
         if self._state:
             for screen in self.screens:
@@ -173,6 +175,10 @@ class Qtile(CommandObject):
         if self.config.reconfigure_screens:
             hook.subscribe.screen_change(self.cmd_reconfigure_screens)
 
+        # If user wants resume hooks we need to add a dbus rule
+        if "resume" in hook.subscriptions:
+            subscribe_for_resume_events()
+
         if initial:
             hook.fire("startup_complete")
 
@@ -240,7 +246,7 @@ class Qtile(CommandObject):
         if self._stopped_event is not None:
             self._stopped_event.set()
 
-    def dump_state(self, buf) -> None:
+    def dump_state(self, buf: Any) -> None:
         try:
             pickle.dump(QtileState(self), buf, protocol=0)
         except:  # noqa: E722
@@ -257,7 +263,7 @@ class Qtile(CommandObject):
         try:
             self.config.load()
         except Exception as error:
-            logger.error("Configuration error: {}".format(error))
+            logger.exception("Configuration error:")
             send_notification("Configuration error", str(error))
             return
 
@@ -299,8 +305,8 @@ class Qtile(CommandObject):
         self._finalize_configurables()
         self.core.finalize()
 
-    def _process_screens(self, reloading=False) -> None:
-        current_groups = [s.group for s in self.screens if hasattr(s, "group")]
+    def _process_screens(self, reloading: bool = False) -> None:
+        current_groups = [s.group for s in self.screens]
         screens = []
 
         if hasattr(self.config, "fake_screens"):
@@ -330,7 +336,7 @@ class Qtile(CommandObject):
             if len(self.groups) < i + 1:
                 name = f"autogen_{i + 1}"
                 self.add_group(name)
-                logger.warning(f"Too few groups in config. Added group: {name}")
+                logger.warning("Too few groups in config. Added group: %s", name)
 
             if i < len(current_groups):
                 grp = current_groups[i]
@@ -338,7 +344,18 @@ class Qtile(CommandObject):
                 for grp in self.groups:
                     if not grp.screen:
                         break
+
             reconfigure_gaps = (x, y, w, h) != (scr.x, scr.y, scr.width, scr.height)
+
+            if not hasattr(scr, "group"):
+                # Ensure that this screen actually *has* a group, as it won't get
+                # assigned one during `__init__` because they are created in the config,
+                # where the groups also are. This lets us type `Screen.group` as
+                # `_Group` rather than `_Group | None` which would need lots of other
+                # changes to check for `None`s, and conceptually all screens should have
+                # a group anyway.
+                scr.group = grp
+
             scr._configure(self, i, x, y, w, h, grp, reconfigure_gaps=reconfigure_gaps)
             screens.append(scr)
 
@@ -375,9 +392,7 @@ class Qtile(CommandObject):
     def process_key_event(self, keysym: int, mask: int) -> None:
         key = self.keys_map.get((keysym, mask), None)
         if key is None:
-            logger.debug(
-                "Ignoring unknown keysym: {keysym}, mask: {mask}".format(keysym=keysym, mask=mask)
-            )
+            logger.debug("Ignoring unknown keysym: %s, mask: %s", keysym, mask)
             return
 
         if isinstance(key, KeyChord):
@@ -389,8 +404,8 @@ class Qtile(CommandObject):
                         (cmd.selectors, cmd.name, cmd.args, cmd.kwargs)
                     )
                     if status in (interface.ERROR, interface.EXCEPTION):
-                        logger.error("KB command error %s: %s" % (cmd.name, val))
-            if self.chord_stack and (self.chord_stack[-1].mode == "" or key.key == "Escape"):
+                        logger.error("KB command error %s: %s", cmd.name, val)
+            if self.chord_stack and (not self.chord_stack[-1].mode or key.key == "Escape"):
                 self.cmd_ungrab_chord()
             return
 
@@ -421,7 +436,7 @@ class Qtile(CommandObject):
     def grab_chord(self, chord: KeyChord) -> None:
         self.chord_stack.append(chord)
         if self.chord_stack:
-            hook.fire("enter_chord", chord.mode)
+            hook.fire("enter_chord", chord.name)
 
         self.ungrab_keys()
         for key in chord.submappings:
@@ -456,12 +471,12 @@ class Qtile(CommandObject):
         for key in self.config.keys:
             self.grab_key(key)
 
-    def grab_button(self, button: Click | Drag) -> None:
+    def grab_button(self, button: Mouse) -> None:
         """Grab the given mouse button event"""
         try:
             button.modmask = self.core.grab_button(button)
         except utils.QtileError:
-            logger.warning(f"Unknown modifier(s): {button.modifiers}")
+            logger.warning("Unknown modifier(s): %s", button.modifiers)
             return
         if button.button_code not in self.mouse_map:
             self.mouse_map[button.button_code] = []
@@ -507,7 +522,7 @@ class Qtile(CommandObject):
             raise ValueError("Can't delete all groups.")
         if name in self.groups_map.keys():
             group = self.groups_map[name]
-            if group.screen and hasattr(group.screen, "previous_group"):
+            if group.screen and group.screen.previous_group:
                 target = group.screen.previous_group
             else:
                 target = group.get_previous_group()
@@ -593,7 +608,7 @@ class Qtile(CommandObject):
         self,
         reserved_space: tuple[int, int, int, int],  # [left, right, top, bottom]
         screen: Screen,
-    ):
+    ) -> None:
         """
         Free up space that has previously been reserved at the edge(s) of a screen.
         """
@@ -712,14 +727,14 @@ class Qtile(CommandObject):
                     if i.check(self):
                         status, val = self.server.call((i.selectors, i.name, i.args, i.kwargs))
                         if status in (interface.ERROR, interface.EXCEPTION):
-                            logger.error("Mouse command error %s: %s" % (i.name, val))
+                            logger.error("Mouse command error %s: %s", i.name, val)
                         handled = True
             elif isinstance(m, Drag):
                 if m.start:
                     i = m.start
                     status, val = self.server.call((i.selectors, i.name, i.args, i.kwargs))
                     if status in (interface.ERROR, interface.EXCEPTION):
-                        logger.error("Mouse command error %s: %s" % (i.name, val))
+                        logger.error("Mouse command error %s: %s", i.name, val)
                         continue
                 else:
                     val = (0, 0)
@@ -751,7 +766,7 @@ class Qtile(CommandObject):
                         (i.selectors, i.name, i.args + (rx + dx, ry + dy), i.kwargs)
                     )
                     if status in (interface.ERROR, interface.EXCEPTION):
-                        logger.error("Mouse command error %s: %s" % (i.name, val))
+                        logger.error("Mouse command error %s: %s", i.name, val)
 
     def warp_to_screen(self) -> None:
         if self.current_screen:
@@ -787,7 +802,7 @@ class Qtile(CommandObject):
         elif name == "widget":
             return False, list(self.widgets_map.keys())
         elif name == "bar":
-            return False, [x.position for x in self.current_screen.gaps]
+            return False, [x.position for x in self.current_screen.gaps if isinstance(x, bar.Bar)]
         elif name == "window":
             windows: list[str | int]
             windows = [
@@ -812,11 +827,13 @@ class Qtile(CommandObject):
             if sel is None:
                 return self.current_group.layout
             else:
-                return utils.lget(self.current_group.layouts, sel)
+                return lget(self.current_group.layouts, int(sel))
         elif name == "widget":
             return self.widgets_map.get(sel)  # type: ignore
         elif name == "bar":
-            return getattr(self.current_screen, sel)  # type: ignore
+            gap = getattr(self.current_screen, sel)  # type: ignore
+            if isinstance(gap, bar.Bar):
+                return gap
         elif name == "window":
             if sel is None:
                 return self.current_window
@@ -832,40 +849,40 @@ class Qtile(CommandObject):
             if sel is None:
                 return self.current_screen
             else:
-                return utils.lget(self.screens, sel)
+                return lget(self.screens, int(sel))
         elif name == "core":
             return self.core
         return None
 
-    def call_soon(self, func: Callable, *args) -> asyncio.Handle:
+    def call_soon(self, func: Callable, *args: Any) -> asyncio.Handle:
         """A wrapper for the event loop's call_soon which also flushes the core's
         event queue after func is called."""
 
-        def f():
+        def f() -> None:
             func(*args)
             self.core.flush()
 
         return self._eventloop.call_soon(f)
 
-    def call_soon_threadsafe(self, func: Callable, *args) -> asyncio.Handle:
+    def call_soon_threadsafe(self, func: Callable, *args: Any) -> asyncio.Handle:
         """Another event loop proxy, see `call_soon`."""
 
-        def f():
+        def f() -> None:
             func(*args)
             self.core.flush()
 
         return self._eventloop.call_soon_threadsafe(f)
 
-    def call_later(self, delay, func: Callable, *args) -> asyncio.TimerHandle:
+    def call_later(self, delay: int, func: Callable, *args: Any) -> asyncio.TimerHandle:
         """Another event loop proxy, see `call_soon`."""
 
-        def f():
+        def f() -> None:
             func(*args)
             self.core.flush()
 
         return self._eventloop.call_later(delay, f)
 
-    def run_in_executor(self, func: Callable, *args):
+    def run_in_executor(self, func: Callable, *args: Any) -> asyncio.Future:
         """A wrapper for running a function in the event loop's default
         executor."""
         return self._eventloop.run_in_executor(None, func, *args)
@@ -917,15 +934,15 @@ class Qtile(CommandObject):
         """
         return {i.name: i.info() for i in self.groups}
 
-    def cmd_display_kb(self, *args) -> str:
+    def cmd_display_kb(self) -> str:
         """Display table of key bindings"""
 
         class FormatTable:
-            def __init__(self):
-                self.max_col_size = []
-                self.rows = []
+            def __init__(self) -> None:
+                self.max_col_size: list[int] = []
+                self.rows: list[list[str]] = []
 
-            def add(self, row):
+            def add(self, row: list[str]) -> None:
                 n = len(row) - len(self.max_col_size)
                 if n > 0:
                     self.max_col_size += [0] * n
@@ -934,23 +951,23 @@ class Qtile(CommandObject):
                         self.max_col_size[i] = len(f)
                 self.rows.append(row)
 
-            def getformat(self):
+            def getformat(self) -> tuple[str, int]:
                 format_string = " ".join(
                     "%-{0:d}s".format(max_col_size + 2) for max_col_size in self.max_col_size
                 )
                 return format_string + "\n", len(self.max_col_size)
 
-            def expandlist(self, list, n):
-                if not list:
+            def expandlist(self, list_: list[str], n: int) -> list[str]:
+                if not list_:
                     return ["-" * max_col_size for max_col_size in self.max_col_size]
-                n -= len(list)
+                n -= len(list_)
                 if n > 0:
-                    list += [""] * n
-                return list
+                    list_ += [""] * n
+                return list_
 
-            def __str__(self):
-                format, n = self.getformat()
-                return "".join([format % tuple(self.expandlist(row, n)) for row in self.rows])
+            def __str__(self) -> str:
+                format_, n = self.getformat()
+                return "".join(format_ % tuple(self.expandlist(row, n)) for row in self.rows)
 
         result = FormatTable()
         result.add(["Mode", "KeySym", "Mod", "Command", "Desc"])
@@ -974,23 +991,23 @@ class Qtile(CommandObject):
                     ]
                 )
                 rows.append(
-                    (
+                    [
                         mode,
                         name,
                         modifiers,
                         "{:s}({:s})".format(k.commands[0].name, allargs),
                         k.desc,
-                    )
+                    ]
                 )
                 return
             if isinstance(k, KeyChord):
-                new_mode_s = k.mode if k.mode else "<unnamed>"
+                new_mode_s = k.name if k.name else "<unnamed>"
                 new_mode = (
-                    k.mode
+                    k.name
                     if mode == "<root>"
-                    else "{}>{}".format(mode, k.mode if k.mode else "_")
+                    else "{}>{}".format(mode, k.name if k.name else "_")
                 )
-                rows.append((mode, name, modifiers, "", "Enter {:s} mode".format(new_mode_s)))
+                rows.append([mode, name, modifiers, "", "Enter {:s} mode".format(new_mode_s)])
                 for s in k.submappings:
                     walk_binding(s, new_mode)
                 return
@@ -1072,7 +1089,7 @@ class Qtile(CommandObject):
         ]
         return lst
 
-    def cmd_simulate_keypress(self, modifiers, key) -> None:
+    def cmd_simulate_keypress(self, modifiers: list[str], key: str) -> None:
         """Simulates a keypress on the focused window.
 
         Parameters
@@ -1112,7 +1129,7 @@ class Qtile(CommandObject):
         try:
             self.config.load()
         except Exception as error:
-            logger.error("Preventing restart because of a configuration error: {}".format(error))
+            logger.exception("Preventing restart because of a configuration error:")
             send_notification("Configuration error", str(error))
             return
         self.restart()
@@ -1140,7 +1157,7 @@ class Qtile(CommandObject):
             args = ["/bin/sh", "-c", cmd]
 
         if shutil.which(to_lookup) is None:
-            logger.error("couldn't find `{}`".format(to_lookup))
+            logger.error("couldn't find `%s`", to_lookup)
             return -1
 
         r, w = os.pipe()
@@ -1300,7 +1317,7 @@ class Qtile(CommandObject):
         """
         mb = self.widgets_map.get(widget)
         if not mb:
-            logger.error("No widget named '{0:s}' present.".format(widget))
+            logger.error("No widget named '%s' present.", widget)
             return
 
         mb.start_input(prompt, self.find_window, "window", strict_completer=True)
@@ -1335,7 +1352,7 @@ class Qtile(CommandObject):
 
         mb = self.widgets_map.get(widget)
         if not mb:
-            logger.error("No widget named '{0:s}' present.".format(widget))
+            logger.error("No widget named '%s' present.", widget)
             return
 
         mb.start_input(prompt, self.move_to_group, "group", strict_completer=True)
@@ -1351,16 +1368,16 @@ class Qtile(CommandObject):
             Name of the prompt widget (default: "prompt")
         """
 
-        def f(group):
+        def f(group: str) -> None:
             if group:
                 try:
                     self.groups_map[group].cmd_toscreen()
                 except KeyError:
-                    logger.warning("No group named '{0:s}' present.".format(group))
+                    logger.warning("No group named '%s' present.", group)
 
         mb = self.widgets_map.get(widget)
         if not mb:
-            logger.error("No widget named '{0:s}' present.".format(widget))
+            logger.error("No widget named '%s' present.", widget)
             return
 
         mb.start_input(prompt, f, "group", strict_completer=True)
@@ -1376,14 +1393,14 @@ class Qtile(CommandObject):
             Name of the prompt widget (default: "prompt")
         """
 
-        def f(name):
+        def f(name: str) -> None:
             self.current_group.cmd_set_label(name or None)
 
         try:
             mb = self.widgets_map[widget]
             mb.start_input(prompt, f, allow_empty_input=True)
         except KeyError:
-            logger.error("No widget named '{0:s}' present.".format(widget))
+            logger.error("No widget named '%s' present.", widget)
 
     def cmd_spawncmd(
         self,
@@ -1413,7 +1430,7 @@ class Qtile(CommandObject):
             this dict, the command it maps to will be executed instead.
         """
 
-        def f(args):
+        def f(args: str) -> None:
             if args:
                 if aliases and args in aliases:
                     args = aliases[args]
@@ -1423,7 +1440,7 @@ class Qtile(CommandObject):
             mb = self.widgets_map[widget]
             mb.start_input(prompt, f, complete)
         except KeyError:
-            logger.error("No widget named '{0:s}' present.".format(widget))
+            logger.error("No widget named '%s' present.", widget)
 
     def cmd_qtilecmd(
         self,
@@ -1446,7 +1463,7 @@ class Qtile(CommandObject):
             "xmessage")
         """
 
-        def f(cmd):
+        def f(cmd: str) -> None:
             if cmd:
                 # c here is used in eval() below
                 q = QtileCommandInterface(self)
@@ -1461,8 +1478,8 @@ class Qtile(CommandObject):
                     return
                 try:
                     result = eval("c.{0:s}".format(cmd))
-                except (CommandError, CommandException, AttributeError) as err:
-                    logger.error(err)
+                except (CommandError, CommandException, AttributeError):
+                    logger.exception("Command errored:")
                     result = None
                 if result is not None:
                     from pprint import pformat
@@ -1474,7 +1491,7 @@ class Qtile(CommandObject):
 
         mb = self.widgets_map[widget]
         if not mb:
-            logger.error("No widget named {0:s} present.".format(widget))
+            logger.error("No widget named %s present.", widget)
             return
         mb.start_input(prompt, f, "qshell")
 
@@ -1497,7 +1514,7 @@ class Qtile(CommandObject):
         match_args: dict[str, Any],
         rule_args: dict[str, Any],
         min_priorty: bool = False,
-    ):
+    ) -> int | None:
         """Add a dgroup rule, returns rule_id needed to remove it
 
         Parameters
@@ -1511,7 +1528,7 @@ class Qtile(CommandObject):
         """
         if not self.dgroups:
             logger.warning("No dgroups created")
-            return
+            return None
 
         match = Match(**match_args)
         rule = Rule([match], **rule_args)
@@ -1538,7 +1555,7 @@ class Qtile(CommandObject):
                 bar.show(not bar.is_show())
                 self.current_group.layout_all()
             else:
-                logger.warning("Not found bar in position '%s' for hide/show." % position)
+                logger.warning("Not found bar in position '%s' for hide/show.", position)
         elif position == "all":
             screen = self.current_screen
             is_show = None
@@ -1552,15 +1569,14 @@ class Qtile(CommandObject):
             else:
                 logger.warning("Not found bar for hide/show.")
         else:
-            logger.warning("Invalid position value:{0:s}".format(position))
+            logger.warning("Invalid position value:%s", position)
 
     def cmd_get_state(self) -> str:
         """Get pickled state for restarting qtile"""
         buf = io.BytesIO()
         self.dump_state(buf)
         state = buf.getvalue().decode(errors="backslashreplace")
-        logger.debug("State = ")
-        logger.debug("".join(state.split("\n")))
+        logger.debug("State = %s", state)
         return state
 
     def cmd_tracemalloc_toggle(self) -> None:
