@@ -18,7 +18,7 @@
 from libqtile.layout.base import Layout
 
 
-class _BspNode():
+class _BspNode:
     def __init__(self, parent=None):
         self.parent = parent
         self.children = []
@@ -147,17 +147,27 @@ class Bsp(Layout):
         Key([mod, "shift"], "n", lazy.layout.normalize()),
         Key([mod], "Return", lazy.layout.toggle_split()),
     """
+
     defaults = [
-        ("name", "bsp", "Name of this layout."),
-        ("border_focus", "#881111", "Border colour for the focused window."),
-        ("border_normal", "#220000", "Border colour for un-focused windows."),
+        ("border_focus", "#881111", "Border colour(s) for the focused window."),
+        ("border_normal", "#220000", "Border colour(s) for un-focused windows."),
         ("border_width", 2, "Border width."),
+        ("border_on_single", False, "Draw border when there is only one window."),
+        (
+            "margin_on_single",
+            None,
+            "Margin when there is only one window (int or list of ints [N E S W], 'None' to use 'margin' value).",
+        ),
         ("margin", 0, "Margin of the layout (int or list of ints [N E S W])."),
-        ("ratio", 1.6,
-         "Width/height ratio that defines the partition direction."),
+        ("ratio", 1.6, "Width/height ratio that defines the partition direction."),
         ("grow_amount", 10, "Amount by which to grow a window/column."),
         ("lower_right", True, "New client occupies lower or right subspace."),
         ("fair", True, "New clients are inserted in the shortest branch."),
+        (
+            "wrap_clients",
+            False,
+            "Whether client list should be wrapped when using ``next`` and ``previous`` commands.",
+        ),
     ]
 
     def __init__(self, **config):
@@ -165,6 +175,8 @@ class Bsp(Layout):
         self.add_defaults(Bsp.defaults)
         self.root = _BspNode()
         self.current = self.root
+        if self.margin_on_single is None:
+            self.margin_on_single = self.margin
 
     def clone(self, group):
         c = Layout.clone(self, group)
@@ -172,10 +184,11 @@ class Bsp(Layout):
         c.current = c.root
         return c
 
+    def get_windows(self):
+        return list(self.root.clients())
+
     def info(self):
-        return dict(
-            name=self.name,
-            clients=[c.name for c in self.root.clients()])
+        return dict(name=self.name, clients=[c.name for c in self.root.clients()])
 
     def get_node(self, client):
         for node in self.root:
@@ -204,11 +217,12 @@ class Bsp(Layout):
             self.current = self.root
 
     def configure(self, client, screen_rect):
-        self.root.calc_geom(screen_rect.x, screen_rect.y, screen_rect.width,
-                            screen_rect.height)
+        self.root.calc_geom(screen_rect.x, screen_rect.y, screen_rect.width, screen_rect.height)
         node = self.get_node(client)
         color = self.border_focus if client.has_focus else self.border_normal
-        border = 0 if node is self.root else self.border_width
+        border = 0 if node is self.root and not self.border_on_single else self.border_width
+        margin = self.margin_on_single if node is self.root else self.margin
+
         if node is not None:
             client.place(
                 node.x,
@@ -217,7 +231,8 @@ class Bsp(Layout):
                 node.h - 2 * border,
                 border,
                 color,
-                margin=self.margin)
+                margin=margin,
+            )
         client.unhide()
 
     def cmd_toggle_split(self):
@@ -232,27 +247,31 @@ class Bsp(Layout):
         clients = list(self.root.clients())
         return clients[-1] if len(clients) else None
 
-    def focus_next(self, client):
+    def focus_next(self, client, wrap=False):
         clients = list(self.root.clients())
         if client in clients:
             idx = clients.index(client)
-            if idx + 1 < len(clients):
-                return clients[idx + 1]
+            if not wrap and idx + 1 < len(clients):
+                return clients[(idx + 1)]
+            elif wrap:
+                return clients[(idx + 1) % len(clients)]
 
-    def focus_previous(self, client):
+    def focus_previous(self, client, wrap=False):
         clients = list(self.root.clients())
         if client in clients:
             idx = clients.index(client)
-            if idx > 0:
-                return clients[idx - 1]
+            if not wrap and idx > 0:
+                return clients[(idx - 1)]
+            elif wrap:
+                return clients[(idx - 1) % len(clients)]
 
     def cmd_next(self):
-        client = self.focus_next(self.current)
+        client = self.focus_next(self.current.client, wrap=self.wrap_clients)
         if client:
             self.group.focus(client, True)
 
     def cmd_previous(self):
-        client = self.focus_previous(self.current)
+        client = self.focus_previous(self.current.client, wrap=self.wrap_clients)
         if client:
             self.group.focus(client, True)
 
@@ -417,8 +436,7 @@ class Bsp(Layout):
         parent = child.parent
         while parent:
             if parent.split_horizontal and child is parent.children[1]:
-                parent.split_ratio = max(5,
-                                         parent.split_ratio - self.grow_amount)
+                parent.split_ratio = max(5, parent.split_ratio - self.grow_amount)
                 self.group.layout_all()
                 break
             child = parent
@@ -429,8 +447,7 @@ class Bsp(Layout):
         parent = child.parent
         while parent:
             if parent.split_horizontal and child is parent.children[0]:
-                parent.split_ratio = min(95,
-                                         parent.split_ratio + self.grow_amount)
+                parent.split_ratio = min(95, parent.split_ratio + self.grow_amount)
                 self.group.layout_all()
                 break
             child = parent
@@ -441,8 +458,7 @@ class Bsp(Layout):
         parent = child.parent
         while parent:
             if not parent.split_horizontal and child is parent.children[1]:
-                parent.split_ratio = max(5,
-                                         parent.split_ratio - self.grow_amount)
+                parent.split_ratio = max(5, parent.split_ratio - self.grow_amount)
                 self.group.layout_all()
                 break
             child = parent
@@ -453,8 +469,7 @@ class Bsp(Layout):
         parent = child.parent
         while parent:
             if not parent.split_horizontal and child is parent.children[0]:
-                parent.split_ratio = min(95,
-                                         parent.split_ratio + self.grow_amount)
+                parent.split_ratio = min(95, parent.split_ratio + self.grow_amount)
                 self.group.layout_all()
                 break
             child = parent
